@@ -4,49 +4,66 @@ Report module
 
 import regex as re
 
-from txtai.extractor import Extractor
+from txtai.pipeline import Extractor, Labels, Similarity, Tokenizer
 
 from ..index import Index
 from ..query import Query
 
-class Report(object):
+from .column import Column
+
+
+class Report:
     """
     Methods to build reports from a series of queries
     """
 
-    def __init__(self, embeddings, db, qa):
+    def __init__(self, embeddings, db, options):
         """
         Creates a new report.
 
         Args:
             embeddings: embeddings index
             db: database connection
-            qa: qa model path
+            options: report options
         """
 
         # Store references to embeddings index and open database cursor
         self.embeddings = embeddings
         self.cur = db.cursor()
 
+        # Report options
+        self.options = options
+
         # Column names
         self.names = []
 
-        # Extractive question-answering model
-        self.extractor = Extractor(self.embeddings, qa if qa else "NeuML/bert-small-cord19qa")
+        self.similarity = (
+            Similarity(options["similarity"]) if "similarity" in options else None
+        )
+        self.labels = Labels(model=self.similarity) if self.similarity else None
 
-    def build(self, queries, topn, threshold, output):
+        # Extractive question-answering model
+        # Determine if embeddings or a custom similarity model should be used to build question context
+        self.extractor = Extractor(
+            self.similarity if self.similarity else self.embeddings,
+            options["qa"] if options["qa"] else "NeuML/bert-small-cord19qa",
+            minscore=options.get("minscore"),
+            mintokens=options.get("mintokens"),
+            context=options.get("context"),
+        )
+
+    def build(self, queries, options, output):
         """
         Builds a report using a list of input queries
 
         Args:
             queries: queries to execute
-            topn: number of documents to return
-            threshold: query match score threshold
+            options: report options
             output: output I/O object
         """
 
         # Default to 50 documents if not specified
-        topn = topn if topn else 50
+        topn = options.get("topn", 50)
 
         for name, config in queries:
             query = config["query"]
@@ -59,7 +76,9 @@ class Report(object):
             self.separator(output)
 
             # Query for best matches
-            results = Query.search(self.embeddings, self.cur, query, topn, threshold)
+            results = Query.search(
+                self.embeddings, self.cur, query, topn, options.get("threshold")
+            )
 
             # Generate highlights section
             self.section(output, "Highlights")
@@ -96,7 +115,9 @@ class Report(object):
         for highlight in Query.highlights(results, topn):
             # Get matching article
             uid = [article for _, _, article, text in results if text == highlight][0]
-            self.cur.execute("SELECT Authors, Reference FROM articles WHERE id = ?", [uid])
+            self.cur.execute(
+                "SELECT Authors, Reference FROM articles WHERE id = ?", [uid]
+            )
             article = self.cur.fetchone()
 
             # Write out highlight row
@@ -117,17 +138,24 @@ class Report(object):
         _, query, _ = metadata
 
         # Retrieve list of documents
-        documents = Query.all(self.cur) if query == "*" else Query.documents(results, topn)
+        documents = (
+            Query.all(self.cur) if query == "*" else Query.documents(results, topn)
+        )
 
         # Collect matching rows
         rows = []
 
-        for uid in documents:
+        for x, uid in enumerate(documents):
             # Get article metadata
-            self.cur.execute("SELECT Published, Title, Reference, Publication, Source, Design, Size, Sample, Method, Entry " +
-                             "FROM articles WHERE id = ?", [uid])
+            self.cur.execute(
+                "SELECT Published, Title, Reference, Publication, Source, Entry, Id FROM articles WHERE id = ?",
+                [uid],
+            )
             article = self.cur.fetchone()
 
+            if x and x % 100 == 0:
+                print(f"Processed {x} documents", end="\r")
+
             # Calculate derived fields
             calculated = self.calculate(uid, metadata)
 
@@ -160,34 +188,63 @@ class Report(object):
         # Parse column parameters
         fields, params = self.params(metadata)
 
-        # Stores embedding query only and full QA column definitions
-        queries, questions = [], []
+        # Different type of calculations
+        #  1. Similarity query
+        #  2. Extractor query (similarity + question)
+        #  3. Question-answering on other field
+        queries, extractions, questions = [], [], []
 
         # Retrieve indexed document text for article
         sections = self.sections(uid)
         texts = [text for _, text in sections]
 
-        for name, query, question, snippet, _, _, matches in params:
-            if matches:
+        for name, query, question, snippet, _, _, matches, _ in params:
+            if query.startswith("$"):
+                questions.append((name, query.replace("$", ""), question, snippet))
+            elif matches:
                 queries.append((name, query, matches))
             else:
-                questions.append((name, query, question, snippet))
+                extractions.append((name, query, question, snippet))
+
+        # Run all extractor queries against document text
+        results = self.extractor.query([query for _, query, _ in queries], texts)
 
         # Only execute embeddings queries for columns with matches set
-        for name, query, matches in queries:
-            # Run query
-            topn = self.extractor.query([query], texts)[0]
+        for x, (name, query, matches) in enumerate(queries):
+            if results[x]:
+                # Get topn text matches
+                topn = [text for _, text, _ in results[x]][:matches]
 
-            # Get topn text matches
-            topn = [text for _, text, _ in topn][:matches]
-
-            # Join results into String and return
-            fields[name] = "\n\n".join([self.resolve(params, sections, uid, name, value) for value in topn])
+                # Join results into String and return
+                value = [
+                    self.resolve(params, sections, uid, name, value) for value in topn
+                ]
+                fields[name] = "\n\n".join(value) if value else None
+            else:
+                fields[name] = None
 
         # Add extraction fields
-        for name, value in self.extractor(questions, texts):
+        for name, value in self.extractor(extractions, texts):
             # Resolves the full value based on column parameters
-            fields[name] = self.resolve(params, sections, uid, name, value) if value else ""
+            fields[name] = (
+                self.resolve(params, sections, uid, name, value) if value else ""
+            )
+
+        # Add question fields
+        names, qa, contexts, snippets = [], [], [], []
+        for name, query, question, snippet in questions:
+            names.append(name)
+            qa.append(question)
+            contexts.append(fields[query])
+            snippets.append(snippet)
+
+        for name, value in self.extractor.answers(
+            names, qa, contexts, contexts, snippets
+        ):
+            # Resolves the full value based on column parameters
+            fields[name] = (
+                self.resolve(params, sections, uid, name, value) if value else ""
+            )
 
         return fields
 
@@ -219,16 +276,32 @@ class Report(object):
             elif "query" in column:
                 # Query variable substitutions
                 query = self.variables(column["query"], metadata)
-                question = self.variables(column["question"], metadata) if "question" in column else query
+                question = (
+                    self.variables(column["question"], metadata)
+                    if "question" in column
+                    else query
+                )
 
                 # Additional context parameters
-                section = column["section"] if "section" in column else False
-                surround = column["surround"] if "surround" in column else 0
-                matches = column["matches"] if "matches" in column else 0
-                snippet = column["snippet"] if "snippet" in column else False
+                section = column.get("section", False)
+                surround = column.get("surround", 0)
+                matches = column.get("matches", 0)
+                dtype = column.get("dtype")
+                snippet = column.get("snippet", False)
                 snippet = True if section or surround else snippet
 
-                params.append((column["name"], query, question, snippet, section, surround, matches))
+                params.append(
+                    (
+                        column["name"],
+                        query,
+                        question,
+                        snippet,
+                        section,
+                        surround,
+                        matches,
+                        dtype,
+                    )
+                )
 
         return fields, params
 
@@ -267,13 +340,19 @@ class Report(object):
         """
 
         # Retrieve indexed document text for article
-        self.cur.execute(Index.SECTION_QUERY + " AND article = ? ORDER BY id", [uid])
+        self.cur.execute(Index.SECTION_QUERY + " WHERE article = ? ORDER BY id", [uid])
 
         # Get list of document text sections
         sections = []
         for sid, name, text in self.cur.fetchall():
-            if not name or not re.search(Index.SECTION_FILTER, name.lower()):
-                sections.append((sid, text))
+            if (
+                not name
+                or not re.search(Index.SECTION_FILTER, name.lower())
+                or self.options.get("allsections")
+            ):
+                # Check that section has at least 1 token
+                if Tokenizer.tokenize(text):
+                    sections.append((sid, text))
 
         return sections
 
@@ -298,7 +377,7 @@ class Report(object):
 
         # Get all column parameters
         index = [params.index(x) for x in params if x[0] == name][0]
-        _, _, _, _, section, surround, _ = params[index]
+        _, _, _, _, section, surround, _, dtype = params[index]
 
         if value:
             # Find matching section
@@ -313,6 +392,14 @@ class Report(object):
                 elif surround:
                     value = self.surround(uid, sid, surround)
 
+            # Column dtype formatting
+            if dtype == "int":
+                value = Column.integer(value)
+            elif isinstance(dtype, list):
+                value = Column.categorical(self.labels, value, dtype)
+            elif dtype in ["days", "weeks", "months", "years"]:
+                value = Column.duration(value, dtype)
+
         return value
 
     def subsection(self, uid, sid):
@@ -327,7 +414,10 @@ class Report(object):
             full text for matching section
         """
 
-        self.cur.execute("SELECT Text FROM sections WHERE article = ? AND name = (SELECT name FROM sections WHERE id = ?)", [uid, sid])
+        self.cur.execute(
+            "SELECT Text FROM sections WHERE article = ? AND name = (SELECT name FROM sections WHERE id = ?)",
+            [uid, sid],
+        )
         return " ".join([x[0] for x in self.cur.fetchall()])
 
     def surround(self, uid, sid, size):
@@ -343,8 +433,11 @@ class Report(object):
             matching text with surrounding context
         """
 
-        self.cur.execute("SELECT Text FROM sections WHERE article = ? AND id in (SELECT id FROM sections WHERE id >= ? AND id <= ?) AND " + \
-                         "name = (SELECT name FROM sections WHERE id = ?)", [uid, sid - size, sid + size, sid])
+        self.cur.execute(
+            "SELECT Text FROM sections WHERE article = ? AND id in (SELECT id FROM sections WHERE id >= ? AND id <= ?) AND "
+            + "name = (SELECT name FROM sections WHERE id = ?)",
+            [uid, sid - size, sid + size, sid],
+        )
 
         return " ".join([x[0] for x in self.cur.fetchall()])
 
