@@ -8,12 +8,11 @@ import sqlite3
 import sys
 import tempfile
 
+from txtai.pipeline import Tokenizer
 from txtai.vectors import WordVectors
-from txtai.tokenizer import Tokenizer
 
-from .models import Models
 
-class RowIterator(object):
+class RowIterator:
     """
     Iterates over rows in a database query. Allows for multiple iterations.
     """
@@ -54,8 +53,8 @@ class RowIterator(object):
         result = next(self.rows)
         if result is None:
             raise StopIteration
-        else:
-            return result
+
+        return result
 
     def stream(self, dbfile):
         """
@@ -78,18 +77,19 @@ class RowIterator(object):
 
             count += 1
             if count % 1000 == 0:
-                print("Streamed %d documents" % (count), end="\r")
+                print(f"Streamed {count} documents", end="\r")
 
             # Skip documents with no tokens parsed
             if tokens:
                 yield tokens
 
-        print("Iterated over %d total rows" % (count))
+        print(f"Iterated over {count} total rows")
 
         # Free database resources
         db.close()
 
-class Vectors(object):
+
+class Vectors:
     """
     Methods to build a FastText model.
     """
@@ -109,7 +109,9 @@ class Vectors(object):
         tokens = None
 
         # Stream tokens to temp working file
-        with tempfile.NamedTemporaryFile(mode="w", suffix=".txt", delete=False) as output:
+        with tempfile.NamedTemporaryFile(
+            mode="w", suffix=".txt", delete=False
+        ) as output:
             # Save file path
             tokens = output.name
 
@@ -124,33 +126,30 @@ class Vectors(object):
         Builds a word vector model.
 
         Args:
-            path: model path, if None uses default path
+            path: model path
             size: dimensions for fastText model
             mincount: minimum number of times a token must appear in input
-            output: output file path, if None uses default path
+            output: output file path
         """
 
-        # Default path if not provided
-        if not path:
-            path = Models.modelPath()
-
         # Derive path to dbfile
         dbfile = os.path.join(path, "articles.sqlite")
 
         # Stream tokens to temporary file
         tokens = Vectors.tokens(dbfile)
 
-        if not output:
-            # Output file path
-            output = Models.vectorPath("cord19-%dd" % size, True)
-
         # Build word vectors model
         WordVectors.build(tokens, size, mincount, output)
 
         # Remove temporary tokens file
         os.remove(tokens)
 
+
 if __name__ == "__main__":
     # Create vector model
-    Vectors.run(sys.argv[1] if len(sys.argv) > 1 else None, 300, 4,
-                sys.argv[2] if len(sys.argv) > 2 else None)
+    Vectors.run(
+        sys.argv[1] if len(sys.argv) > 1 else None,
+        300,
+        4,
+        sys.argv[2] if len(sys.argv) > 2 else None,
+    )
